<?php

// Check if class already exists

if (!class_exists("WPsite_Stats_BT")):

// AJAX functions for click tracking

add_action( 'wp_ajax_wps_bt_record_impression', 		array('WPsite_Stats_BT', 'recordImpresssion'));
add_action( 'wp_ajax_nopriv_wps_bt_record_impression', 	array('WPsite_Stats_BT', 'recordImpresssion'));
add_action( 'wp_ajax_wps_bt_record_click', 				array('WPsite_Stats_BT', 'recordClick'));
add_action( 'wp_ajax_nopriv_wps_bt_record_click', 		array('WPsite_Stats_BT', 'recordClick'));

/**
 * WPsite_Stats_BT class.
 *
 * This class is used to record all stats for background takeovers.
 */
class WPsite_Stats_BT {

	/**
	 * table_name
	 *
	 * (default value: 'wpsite_background_takeover_stats')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	public static $table_name = 'wpsite_background_takeover_stats';

	/**
	 * Create Table when this class is loaded
	 *
	 * @access public
	 * @return void
	 */
	function __construct() {
		self::createTable();
	}

	/**
	 * Creates the Stats table if it does not exist
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function createTable(){

		global $wpdb;

		$wpdb->query("
			CREATE TABLE IF NOT EXISTS `" . self::getTableName() . "` (
				`id` int(11) NOT NULL AUTO_INCREMENT,
				`date` date NOT NULL,
				`background_takeover_id` int(11) NOT NULL DEFAULT 0,
				`impressions` int(8) NOT NULL DEFAULT 0,
				`left_clicks` int(8) NOT NULL DEFAULT 0,
				`right_clicks` int(8) NOT NULL DEFAULT 0,
				PRIMARY KEY (`id`)
			) ENGINE=InnoDB DEFAULT CHARSET=latin1 AUTO_INCREMENT=1;
		");

	}

	/**
	 * Get the stats for all background takeovers in a timeframe or a single firbar in a timeframe
	 *
	 * @access public
	 * @static
	 * @param mixed $start
	 * @param mixed $end
	 * @param mixed $id (default: null)
	 * @return void
	 */
	static function getStats($start, $end, $id = null, $select = '*') {

		$query = null;

		global $wpdb;

		// All Background Takeovers, All Time

		if ($start == null && $end == null && $id == null) {
			$query = 'SELECT ' . $select . ' FROM ' . self::getTableName();
		}

		// All Background Takeovers, After Date

		else if ($start != null && $end == null && $id == null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `date` = %s', $start);
		}

		// All Background Takeovers, Date Range

		else if ($start != null && $end != null && $id == null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `date` >= %s AND `date` <= %s', $start, $end);
		}

		// Single Background Takeover, All Time

		else if ($start == null && $end == null && $id != null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `background_takeover_id` = %d', $id);
		}

		// Single Background Takeover, Date Range

		else if ($start != null && $end != null && $id != null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `date` >= %s AND `date` <= %s AND `background_takeover_id` = %d', $start, $end, $id);
		}

		// No query was created

		if (!isset($query)) {
			return false;
		}

		$result = $wpdb->get_results($query, 'ARRAY_A');

		return $result;
	}

	/**
	 * Runs everyday and will add a record to table if none exists
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function initialize_background_takeover_stats() {

		global $wpdb;
		$today = date(WPSITE_WALLPAPER_TAKEOVER_DATE_FORMAT);

		include_once(WPSITE_WALLPAPER_TAKEOVER_PLUGIN_DIR . "/include/classes/Ads_DB_WPTO.php");
		$adsMgn = new Ads_DB_WPTO();

		$background_takeovers = $wpdb->get_results("SELECT * FROM `" . $adsMgn->tb_nm . "`");

		$result = null;

		foreach ($background_takeovers as $background_takeover) {

			// Check if entry already exsits

			$query = $wpdb->prepare('SELECT * FROM ' . self::getTableName() . ' WHERE `date` = %s AND `background_takeover_id` = %d', $today, $background_takeover->ad_id);
			$background_takeover_impressions = $wpdb->query($query);

			// Entry does not exist, create a new one and set impressions to 0 and clicks to 0

			if (!isset($background_takeover_impressions) || $background_takeover_impressions == 0) {

				$query = $wpdb->prepare('INSERT INTO ' . self::getTableName() . ' (
					`date`,
					`background_takeover_id`,
					`impressions`,
					`left_clicks`,
					`right_clicks`
					) VALUES (%s, %d, 0, 0, 0)',
					$today,
					$background_takeover->ad_id
				);

				$result = $wpdb->query($query);
			}
		}

		return $result;

	}

	/**
	 * Record an impression made by a background takeover
	 * This function is called via PHP.
	 *
	 * @access public
	 * @static
	 * @param mixed $data (default: null)
	 * @return void
	 */
	static function recordImpresssion($data = null) {

        $data = [];
		$data['background_takeover_id'] = $_POST['background_takeover_id'];

		// Return false if no data is given

		if (!isset($data) || !isset($data['background_takeover_id'])) {
			echo 'Background Takeover impression was NOT able to be recored because the Background Takeover ID was not retrieved.';
			die();
		}

		// Return false if crawler

		if (self::isBot()) {
			echo 'Background Takeover impression was NOT able to be recored because a crawler was detected.';
			die();
		}

		// Return if user is admin or higher

		if (current_user_can('edit_published_posts')) {
			echo 'Background Takeover impression was NOT able to be recored because current user is an admin.';
			die();
		}

		global $wpdb;
		$today = date(WPSITE_WALLPAPER_TAKEOVER_DATE_FORMAT);

		// Check if entry already exsits

		$query = $wpdb->prepare('SELECT * FROM ' . self::getTableName() . ' WHERE `date` = %s AND `background_takeover_id` = %d', $today, $data['background_takeover_id']);
		$background_takeover_impressions = $wpdb->query($query);

		// Entry aleady exists, just add 1

		if (isset($background_takeover_impressions) && $background_takeover_impressions != 0) {

			$query = $wpdb->prepare('UPDATE ' . self::getTableName() . ' SET
				`impressions` = `impressions` + 1
				WHERE `date` = %s AND `background_takeover_id` = %d', $today, $data['background_takeover_id']);

			$result = $wpdb->query($query);

		}

		// Entry does not exist, create a new one and set impressions to 1

		else {
			$query = $wpdb->prepare('INSERT INTO ' . self::getTableName() . ' (
				`date`,
				`background_takeover_id`,
				`impressions`,
				`left_clicks`,
				`right_clicks`
				) VALUES (%s, %d, 1, 0, 0)',
				$today,
				$data['background_takeover_id']
			);

			$result = $wpdb->query($query);
		}

		if ($result) {
			echo 'Background Takeover impression was able to be recored.';
		} else {
			echo 'Background Takeover impression was NOT able to be recored.';
		}

		die(); // necessary for WordPress AJAX calls

	}

	/**
	 * Record a click made by a background takeover
	 * This function is called via AJAX
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function recordClick() {

		$background_takeover_id = $_POST['background_takeover_id'];

		$side = $_POST['side'];

		// Return if no data is given

		if (!isset($background_takeover_id)) {
			echo 'Background Takeover click was NOT able to be recored because the Background Takeover ID was not retrieved.';
			die();
		}

		// Return if user is admin or higher

		if (current_user_can('edit_published_posts')) {
			echo 'Background Takeover click was NOT able to be recored because current user is not authorized to log clicks.';
			die();
		}

		// Return false if crawler

		if (self::isBot()) {
			echo 'Background Takeover click was NOT able to be recored because there is no user.  A crawler was detected';
			die();
		}

		global $wpdb;
		$today = date(WPSITE_WALLPAPER_TAKEOVER_DATE_FORMAT);

		// Check if entry already exsits

		$query = $wpdb->prepare('SELECT * FROM ' . self::getTableName() . ' WHERE `date` = %s AND `background_takeover_id` = %d', $today, $background_takeover_id);
		$background_takeover_clicks = $wpdb->query($query);

		// Entry aleady exists, just add 1

		if (isset($background_takeover_clicks) && $background_takeover_clicks != 0) {

			if ($side == 'left') {
				$query = $wpdb->prepare('UPDATE ' . self::getTableName() . ' SET
					`left_clicks` = `left_clicks` + 1
					WHERE `date` = %s AND `background_takeover_id` = %d', $today, $background_takeover_id);
			} else {
				$query = $wpdb->prepare('UPDATE ' . self::getTableName() . ' SET
					`right_clicks` = `right_clicks` + 1
					WHERE `date` = %s AND `background_takeover_id` = %d', $today, $background_takeover_id);
			}


			$result = $wpdb->query($query);

		}

		// Entry does not exist, create a new one and set impressions to 1

		else {

			if ($side == 'left') {
				$query = $wpdb->prepare('INSERT INTO ' . self::getTableName() . ' (
					`date`,
					`background_takeover_id`,
					`impressions`,
					`left_clicks`,
					`right_clicks`
					) VALUES (%s, %d, 0, 1, 0)',
					$today,
					$background_takeover_id
				);
			} else {
				$query = $wpdb->prepare('INSERT INTO ' . self::getTableName() . ' (
					`date`,
					`background_takeover_id`,
					`impressions`,
					`left_clicks`,
					`right_clicks`
					) VALUES (%s, %d, 0, 0, 1)',
					$today,
					$background_takeover_id
				);
			}

			$result = $wpdb->query($query);
		}

		if ($result) {
			echo 'Background Takeover click was able to be recored.';
		} else {
			echo 'Background Takeover click was NOT able to be recored.';
		}

		die(); // necessary for WordPress AJAX calls
	}

	/**
	 * Deletes the stats for a background takeover when the background takeover is deleted
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function deleteStats($background_takeover_id) {

		// Return false if background_takeover_id is not set

		if (!isset($background_takeover_id)) {
			return false;
		}

		global $wpdb;

		// Check if entry already exsits

		$query = $wpdb->prepare('DELETE FROM ' . self::getTableName() . ' WHERE `background_takeover_id` = %d', $background_takeover_id);
		$result = $wpdb->query($query);

		return $result;

	}

	/**
	 * Check for BOTs visiting site
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function isBot() {

		if (isset($_SERVER['HTTP_USER_AGENT']) && preg_match('/bot|crawl|slurp|spider/i', $_SERVER['HTTP_USER_AGENT'])) {
			return true;
		} else {
			return false;
		}

	}

	/**
	 * Returns the proper table name based off of site ID
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function getTableName(){

		global $wpdb;

		return $wpdb->prefix . self::$table_name;
	}

}

endif;